#ifndef __MY_TYPES_H__
#define __MY_TYPES_H__

/****************************************************************************
	my types -- common type declarations I like to use
	
	inspired by _Writing Solid Code_ by Steve Maguire

	I'm probably duplicating something in the std library anyway
	
*****************************************************************************/

/* 	Status flag useful for function error return values.
	Maybe it should enumerate? */
typedef unsigned short int flag;
typedef char byte;

#endif
